#!/usr/bin/env python3

# Experimenting with gpt
#
#-----------------------------------------------------------------
# Copyright 2023 Tanel Tammet (tanel.tammet@gmail.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#-------------------------------------------------------------------

# ==== standard libraries ====

import sys
import json
import http.client

# ==== import other source files ====


# ======= llm configuration ===

secrets_file="secrets.js"

gpt2="code-davinci-002"         # text-davinci-002 code-davinci-002 babbage-002 
gpt3="gpt-3.5-turbo-1106"  # gpt-3.5-turbo-instruct
gpt4="gpt-4-1106-preview"  # gpt-4  gpt-4-32k 

temperature=0
seed=1234
max_tokens=200

gpt_model=gpt3

# ===============================

gpt_prompt="""Simplify, maximally shorten and split the sentence after colon to shortest possible separate subsentences, to make it understandable for children. 
Replace pronouns like 'they','it','he' etc in the result with nouns and proper nouns present in the result, like 'Birds can fly. Birds have feathers' instead of 
'Birds can fly. They have feathers.'. Prepend the character star * to concrete objects and the character plus + to general nouns, 
like 'Birds+ can fly. A stork* ate a frog*.':  """

#gpt_model="gpt-3.5-turbo"
gpt_prompt="""Simplify, maximally shorten and split the sentence after colon to shortest possible separate subsentences, to make it understandable for children. 
Replace pronouns like 'they','it','he','she' in the result with nouns and proper nouns present in the result, 
like 'Birds can fly. Birds have feathers' instead of 
'Birds can fly. They have feathers':  """

gpt_prompt="""Simplify, maximally shorten and split the sentence after colon to shortest possible separate subsentences, to make it understandable for children. 
Replace pronouns like 'they','it','he','she' in the result with nouns and proper nouns present in the result. For example,
instead of 'Birds can fly. They have feathers.' say 'Birds can fly. Birds have feathers.' : """


min_simplification_length = 6

# ======= globals used and changed during work ===

#constant_nr=0 # new constants created add a numeric suffic, incremented
#definition_nr=0 # new definitions created add a numeric suffic, incremented

# ========= code ===================

def main():
  #print(sys.argv)
  gptversion=gpt3
  pfile=""
  texts=[]
  files=[]    
  nextprompt=False
  max_tokens=None
  # parse command line
  for el in sys.argv[1:]:
    if el in ["-sys","-s","--sys","--s"]:
      nextprompt=True
    else:
      if nextprompt:
        pfile=el
        nextprompt=False
      else:  
        if el=="3" or el=="3.5":
          gptversion=gpt3
        elif el=="2":
          gptversion=gpt2  
        elif el=="4":
          gptversion=gpt4  
        elif el.strip().isnumeric():
          max_tokens=int(el.strip())
        elif len(el)<20 and " " not in el:
          try:
            f=open(el, "r")
            onetext=f.read().strip()
            f.close()
            texts.append(onetext)
          except:
            show_error("could not read prompt file "+el)  
        else:
          texts.append(el)  
  # make a prompt         
  prompt=" ".join(texts).strip()    
  # make a sysprompt
  sysprompt=""    
  if pfile:
    try:
      f=open(pfile, "r")
      sysprompt=f.read().strip()
      f.close()
    except:
      show_error("could not read sysprompt file "+pfile)
  if not prompt:
    show_error("no prompt given")    
  print("gpt:",gptversion)
  print("sysprompt:",sysprompt)
  print("prompt:",prompt)
  result=call_gpt(gptversion,prompt,sysprompt,max_tokens)
  print("result:",result)
  

# ========= llm connection =========


def call_gpt(gptversion,sentences,sysprompt,max_tokens):
  try:
    sf=open(secrets_file,"r")
    txt=sf.read()
  except:
    return "Could not read file containing gpt api key: "+str(secrets_file)
  try:  
    data=json.loads(txt)
  except:
    return "Could not parse json text containing gpt api key in: "+str(secrets_file)  
  if "gpt_key" not in data or not (data["gpt_key"]):
    return "Could not find gpt api key in: "+str(secrets_file)
  else:    
    key=data["gpt_key"]
  # key found ok    
  #sentences="A fork is a tool you use in the kitchen or when you eat."
  messages=[]
  if sysprompt:
    message1={"role": "system", "content": sysprompt}
    messages.append(message1)   
  message2={"role": "user", "content": sentences}
  messages.append(message2)  

  call={
     "model": gptversion,
     "messages": messages, #[message1,message2],
     "seed": seed,
     #"logprobs": True,
     "temperature": temperature
  }
  if max_tokens:
    call["max_tokens"]=max_tokens

  #debug_print("gpt call",call)
  calltxt=json.dumps(call) 
  debug_print("gpt call:",calltxt)
  #return

  host = "api.openai.com"
  conn = http.client.HTTPSConnection(host)
  conn.request("POST", "/v1/chat/completions", calltxt,
               headers={
    "Host": host, "Content-Type": "application/json", "Authorization": "Bearer "+key 
  })
  response = conn.getresponse()
  if response.status!=200 or response.reason!="OK":
    show_error("gpt responded with error "+str(response.status)+" "+str(response.reason))
    sys.exit(0)
  rawdata = response.read()
  try:
    data=json.loads(rawdata)
  except KeyboardInterrupt:
    raise  
  except:
    show_error("gpt response is not a correct json: "+  str(rawdata))
    sys.exit(0)
  if "choices" not in data:
    show_error("gpt response does not contain choices")
    sys.exit(0)
  debug_print("gpt response:",data)  
  part=data["choices"]  
  res=""
  for el in part:
    if "message" in el:
      msg=el["message"]
      if "content" in msg:
        tmp=msg["content"]
        if len(tmp)>2 and tmp[0] in ["\"","'"] and tmp[-1] in ["\"","'"]:
          tmp=tmp[1:-1]
        tmp2=tmp.split("\n")
        if len(tmp2)>1:
          tmp3=""
          for line in tmp2:
            if len(line)>3 and (line[0].isnumeric() or line[0] in ["*","-"]) and line[1] in [".",":"," "]:
              tmp3+=line[2:]+" "
            else:
              tmp3+=line+" "  
          tmp=tmp3    
        res+=tmp
  conn.close()
  #debug_print("res",res)  
  return res

def debug_print(a,b):  
  print(a,b)

def show_error(a):
  print(a)
  exit(0)  

if __name__ == "__main__":
  main()

# =========== the end ==========
